"use strict";
var JsonExcludePropertyFilter = (function () {
    function JsonExcludePropertyFilter(properties) {
        var _this = this;
        this.apply = function (source) {
            if (_this._properties.length) {
                for (var _i = 0, _a = _this._properties; _i < _a.length; _i++) {
                    var rule = _a[_i];
                    _this._exclude(rule, source);
                }
                return source;
            }
            return source;
        };
        this._properties = properties;
    }
    JsonExcludePropertyFilter.prototype._exclude = function (rule, source) {
        if (rule.match(JsonExcludePropertyFilter.ALL_PROPERTIES_REGEX)) {
            this._excludeProperties(rule, source);
        }
        else if (rule.match(JsonExcludePropertyFilter.ALL_ELEMENT_PROPERTIES_REGEX)) {
            this._excludeRootProperties(rule, source);
        }
        else {
            this._excludeSpecificPath(rule, source);
        }
    };
    JsonExcludePropertyFilter.prototype._excludeProperty = function (rule, source, path) {
        var pathWithoutArrayIndexStart = this._removeArrayIndexStart(path);
        var regexp = "^" + rule;
        if (pathWithoutArrayIndexStart.match(regexp)) {
            delete source[path];
        }
    };
    JsonExcludePropertyFilter.prototype._excludeProperties = function (rule, source) {
        var formattedRule = rule.substr(0, rule.length - 2);
        for (var path in source) {
            if (path) {
                this._excludeProperty(formattedRule, source, path);
            }
        }
    };
    JsonExcludePropertyFilter.prototype._excludeRootProperty = function (rule, path, source) {
        var pathWithoutIndex = this._removeArrayIndex(path);
        if (rule === JsonExcludePropertyFilter.STRING_EMPTY) {
            var pathWithoutArrayIndexStart = this._removeArrayIndexStart(path);
            var splittedPath = pathWithoutArrayIndexStart.split(JsonExcludePropertyFilter.PATH_SEPARATOR);
            if (splittedPath.length === 1) {
                delete source[path];
            }
        }
        else {
            var regexp = "^" + rule;
            if (pathWithoutIndex.match(regexp)) {
                var pathItems = pathWithoutIndex.split(JsonExcludePropertyFilter.PATH_SEPARATOR);
                var ruleItems = rule.split(JsonExcludePropertyFilter.PATH_SEPARATOR);
                if (pathItems.length === ruleItems.length) {
                    delete source[path];
                }
            }
        }
    };
    JsonExcludePropertyFilter.prototype._excludeRootProperties = function (rule, source) {
        var ruleWithoutRootSymbol = rule.substr(0, rule.length - 1);
        for (var path in source) {
            if (path) {
                this._excludeRootProperty(ruleWithoutRootSymbol, path, source);
            }
        }
    };
    JsonExcludePropertyFilter.prototype._excludeSpecificPath = function (rule, source) {
        var regexp = "^" + rule;
        for (var path in source) {
            if (path) {
                var pathWithoutArrayIndexStart = this._removeArrayIndexStart(path);
                var pathWithoutIndex = this._removeArrayIndex(pathWithoutArrayIndexStart);
                if (pathWithoutIndex.match(regexp)) {
                    var pathWithoutIndexItems = pathWithoutIndex.split(JsonExcludePropertyFilter.PATH_SEPARATOR);
                    var ruleItems = rule.split(".");
                    var pathWithoutIndexItem = pathWithoutIndexItems[ruleItems.length - 1];
                    var ruleItem = ruleItems[ruleItems.length - 1];
                    if (pathWithoutIndexItem === ruleItem) {
                        delete source[path];
                    }
                }
            }
        }
    };
    JsonExcludePropertyFilter.prototype._removeArrayIndex = function (path) {
        return path.replace(JsonExcludePropertyFilter.ARRAY_INDEX, JsonExcludePropertyFilter.STRING_EMPTY);
    };
    JsonExcludePropertyFilter.prototype._removeArrayIndexStart = function (path) {
        return path.replace(JsonExcludePropertyFilter.ARRAY_INDEX_START, JsonExcludePropertyFilter.STRING_EMPTY);
    };
    JsonExcludePropertyFilter.ALL_PROPERTIES_REGEX = /\*\*$/g;
    JsonExcludePropertyFilter.ALL_ELEMENT_PROPERTIES_REGEX = /\*$/g;
    JsonExcludePropertyFilter.ARRAY_INDEX = /\[[0-9]+\]/g;
    JsonExcludePropertyFilter.ARRAY_INDEX_START = /^\[([0-9]+)\]\./;
    JsonExcludePropertyFilter.PATH_SEPARATOR = ".";
    JsonExcludePropertyFilter.STRING_EMPTY = "";
    return JsonExcludePropertyFilter;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = JsonExcludePropertyFilter;
