"use strict";
var JsonIncludePropertyFilter = (function () {
    function JsonIncludePropertyFilter(properties) {
        var _this = this;
        this.apply = function (source) {
            if (_this._properties.length) {
                var destination = new Array();
                for (var _i = 0, _a = _this._properties; _i < _a.length; _i++) {
                    var rule = _a[_i];
                    _this._include(rule, source, destination);
                }
                return destination;
            }
            else {
                return source;
            }
        };
        this._properties = properties;
    }
    JsonIncludePropertyFilter.prototype._cleanupPath = function (path) {
        return path.replace(JsonIncludePropertyFilter.ARRAY_INDEX_START, "");
    };
    JsonIncludePropertyFilter.prototype._include = function (rule, source, destination) {
        if (rule.match(JsonIncludePropertyFilter.ALL_PROPERTIES_REGEX)) {
            this._includeProperties(rule, source, destination);
        }
        else if (rule.match(JsonIncludePropertyFilter.ALL_ELEMENT_PROPERTIES_REGEX)) {
            this._includeRootProperties(rule, source, destination);
        }
        else {
            this._includeSpecificPath(rule, source, destination);
        }
    };
    JsonIncludePropertyFilter.prototype._includeProperty = function (rule, path, value, destination) {
        var cleanPath = this._cleanupPath(path);
        var regexp = "^" + rule;
        if (cleanPath.match(regexp)) {
            destination[path] = value;
        }
    };
    JsonIncludePropertyFilter.prototype._includeProperties = function (rule, source, destination) {
        var formattedRule = rule.substr(0, rule.length - 2);
        for (var path in source) {
            if (path) {
                var value = source[path];
                this._includeProperty(formattedRule, path, value, destination);
            }
        }
    };
    JsonIncludePropertyFilter.prototype._includeRootProperty = function (rule, path, value, source, destination) {
        var pathWithoutIndex = path.replace(JsonIncludePropertyFilter.ARRAY_INDEX, JsonIncludePropertyFilter.STRING_EMPTY);
        if (rule === JsonIncludePropertyFilter.STRING_EMPTY) {
            var cleanPath = this._cleanupPath(path);
            var splittedPath = cleanPath.split(JsonIncludePropertyFilter.PATH_SEPARATOR);
            if (splittedPath.length === 1) {
                destination[path] = value;
            }
        }
        else {
            var regexp = "^" + rule;
            if (pathWithoutIndex.match(regexp)) {
                var pathItems = pathWithoutIndex.split(JsonIncludePropertyFilter.PATH_SEPARATOR);
                var ruleItems = rule.split(JsonIncludePropertyFilter.PATH_SEPARATOR);
                if (pathItems.length === ruleItems.length) {
                    destination[path] = value;
                }
            }
        }
    };
    JsonIncludePropertyFilter.prototype._includeRootProperties = function (rule, source, destination) {
        var ruleWithoutRootSymbol = rule.substr(0, rule.length - 1);
        for (var path in source) {
            if (path) {
                var value = source[path];
                this._includeRootProperty(ruleWithoutRootSymbol, path, value, source, destination);
            }
        }
    };
    JsonIncludePropertyFilter.prototype._includeSpecificPath = function (rule, source, destination) {
        for (var path in source) {
            if (path) {
                var cleanPath = this._cleanupPath(path);
                var pathWithoutIndex = cleanPath.replace(JsonIncludePropertyFilter.ARRAY_INDEX, JsonIncludePropertyFilter.STRING_EMPTY);
                var regexp = "^" + rule;
                if (pathWithoutIndex.match(regexp)) {
                    destination[path] = source[path];
                }
            }
        }
    };
    JsonIncludePropertyFilter.ALL_PROPERTIES_REGEX = /\*\*$/g;
    JsonIncludePropertyFilter.ALL_ELEMENT_PROPERTIES_REGEX = /\*$/g;
    JsonIncludePropertyFilter.ARRAY_INDEX = /\[[0-9]+\]/g;
    JsonIncludePropertyFilter.ARRAY_INDEX_START = /^\[([0-9]+)\]./;
    JsonIncludePropertyFilter.PATH_SEPARATOR = ".";
    JsonIncludePropertyFilter.STRING_EMPTY = "";
    return JsonIncludePropertyFilter;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = JsonIncludePropertyFilter;
